#
# Copyright 2018, 2019 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#
""" The file for testing the gr-modtool scripts """


import os
import shutil
import tempfile
import unittest
import warnings
from os import path
try:
    from pylint.epylint import py_run
    skip_pylint_test = False
except:
    skip_pylint_test = True

try:  # for debugging, full path must be given
    from gnuradio.modtool.core import ModToolNewModule
    from gnuradio.modtool.core import ModToolAdd
    from gnuradio.modtool.core import ModToolDisable
    from gnuradio.modtool.core import ModToolException
    from gnuradio.modtool.core import ModToolMakeYAML
    from gnuradio.modtool.core import ModToolRename
    from gnuradio.modtool.core import ModToolRemove
except:
    from modtool.core import ModToolNewModule
    from modtool.core import ModToolAdd
    from modtool.core import ModToolDisable
    from modtool.core import ModToolException
    from modtool.core import ModToolMakeYAML
    from modtool.core import ModToolRename
    from modtool.core import ModToolRemove


class TestModToolCore(unittest.TestCase):
    """ The tests for the modtool core """

    def __init__(self, *args, **kwargs):
        super(TestModToolCore, self).__init__(*args, **kwargs)
        self.f_add = False
        self.f_newmod = False
        self.srcdir = path.abspath(path.join(path.dirname(
            path.realpath(__file__)), '../templates/gr-newmod'))

    @classmethod
    def setUpClass(cls):
        """ create a temporary directory """
        cls.start_dir = os.getcwd()
        cls.test_dir = tempfile.mkdtemp()

    @classmethod
    def tearDownClass(cls):
        """ remove the directory after the test """
        os.chdir(cls.start_dir)
        shutil.rmtree(cls.test_dir)

    def setUp(self):
        """ create a new module and block before every test """
        try:
            warnings.simplefilter("ignore", ResourceWarning)
            args = {'module_name': 'howto',
                    'directory': self.test_dir,
                    'srcdir': self.srcdir}
            ModToolNewModule(**args).run()
        except (TypeError, ModToolException):
            self.f_newmod = True
        else:
            try:
                args = {'blockname': 'square_ff', 'block_type': 'general',
                        'lang': 'cpp', 'directory': self.test_dir + '/gr-howto',
                        'add_python_qa': True}
                ModToolAdd(**args).run()
            except (TypeError, ModToolException):
                self.f_add = True

    def tearDown(self):
        """ removes the created module """
        # Required, else the new-module directory command
        # in setup will throw exception after first test
        # cannot remove if directory is not created
        if not self.f_newmod:
            os.chdir(self.start_dir)
            rmdir = self.test_dir + '/gr-howto'
            shutil.rmtree(rmdir)

    def test_newmod(self):
        """ Tests for the API function newmod """
        ## Tests for proper exceptions ##
        test_dict = {'directory': self.test_dir,
                     'srcdir': self.srcdir}
        # module name not specified
        self.assertRaises(ModToolException, ModToolNewModule(**test_dict).run)
        test_dict['module_name'] = 'howto'
        # expected module_name as a string instead of dict
        self.assertRaises(TypeError, ModToolNewModule(test_dict).run)
        # directory already exists
        # will not be raised if the command in setup failed
        self.assertRaises(ModToolException, ModToolNewModule(**test_dict).run)

        ## Some tests for checking the created directory, sub-directories and files ##
        test_dict['module_name'] = 'test'
        ModToolNewModule(**test_dict).run()
        module_dir = path.join(self.test_dir, 'gr-test')
        self.assertTrue(path.isdir(module_dir))
        self.assertTrue(path.isdir(path.join(module_dir, 'lib')))
        self.assertTrue(path.isdir(path.join(module_dir, 'python')))
        self.assertTrue(path.isdir(path.join(module_dir, 'include')))
        self.assertTrue(path.isdir(path.join(module_dir, 'docs')))
        self.assertTrue(path.isdir(path.join(module_dir, 'cmake')))
        self.assertTrue(path.exists(path.join(module_dir, 'CMakeLists.txt')))

        ## The check for object instantiation ##
        test_obj = ModToolNewModule(srcdir=self.srcdir)
        # module name not specified
        with self.assertRaises(ModToolException) as context_manager:
            test_obj.run()
        test_obj.info['modname'] = 'howto'
        test_obj.directory = self.test_dir
        # directory already exists
        self.assertRaises(ModToolException, test_obj.run)
        test_obj.info['modname'] = 'test1'
        test_obj.run()
        self.assertTrue(path.isdir(self.test_dir + '/gr-test1'))
        self.assertTrue(path.isdir(self.test_dir + '/gr-test1/lib'))
        self.assertTrue(path.exists(
            self.test_dir + '/gr-test1/CMakeLists.txt'))

    @unittest.skipIf(skip_pylint_test, 'pylint dependency missing, skip test')
    def test_pylint_newmod(self):
        """ Pylint tests for API function newmod """
        module_dir = path.join(self.test_dir, 'gr-test')
        ## pylint tests ##
        python_dir = path.join(module_dir, 'python', 'test')
        py_module = path.join(python_dir, 'mul_ff.py')
        (pylint_stdout, pylint_stderr) = py_run(py_module +
                                                ' --errors-only --disable=E0602', return_std=True)
        print(pylint_stdout.getvalue(), end='')
        py_module = path.join(python_dir, 'qa_mul_ff.py')
        (pylint_stdout, pylint_stderr) = py_run(
            py_module + ' --errors-only', return_std=True)
        print(pylint_stdout.getvalue(), end='')

    def test_add(self):
        """ Tests for the API function add """
        # skip tests if newmod command wasn't successful
        if self.f_newmod:
            raise unittest.SkipTest("setUp for API function 'add' failed")
        module_dir = path.join(self.test_dir, 'gr-howto')
        ## Tests for proper exceptions ##
        test_dict = {}
        test_dict['directory'] = module_dir
        # missing blockname, block_type, lang
        self.assertRaises(ModToolException, ModToolAdd(**test_dict).run)
        test_dict['blockname'] = 'add_ff'
        # missing arguments block_type, lang
        self.assertRaises(ModToolException, ModToolAdd(**test_dict).run)
        test_dict['block_type'] = 'general'
        # missing argument lang
        self.assertRaises(ModToolException, ModToolAdd(**test_dict).run)
        test_dict['lang'] = 'cxx'
        # incorrect language
        self.assertRaises(ModToolException, ModToolAdd(**test_dict).run)
        test_dict['lang'] = 'cpp'
        test_dict['add_cpp_qa'] = 'Wrong'
        # boolean is expected for add_cpp_qa
        self.assertRaises(ModToolException, ModToolAdd(**test_dict).run)
        test_dict['add_cpp_qa'] = True
        test_dict['block_type'] = 'generaleee'
        # incorrect block type
        self.assertRaises(ModToolException, ModToolAdd(**test_dict).run)
        test_dict['block_type'] = 'general'
        test_dict['skip_lib'] = 'fail'
        # boolean value is expected for skip_lib, fails in instantiation
        self.assertRaises(ModToolException, ModToolAdd(**test_dict).run)
        test_dict['skip_lib'] = True
        # missing relevant subdir
        self.assertRaises(ModToolException, ModToolAdd(**test_dict).run)

        ## Some tests for checking the created directory, sub-directories and files ##
        test_dict['skip_lib'] = False
        ModToolAdd(**test_dict).run()
        self.assertTrue(path.exists(
            path.join(module_dir, 'lib', 'qa_add_ff.cc')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'lib', 'add_ff_impl.cc')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'grc', 'howto_add_ff.block.yml')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'include', 'gnuradio', 'howto', 'add_ff.h')))

        ## The check for object instantiation ##
        test_obj = ModToolAdd()
        test_obj.dir = module_dir
        # missing blocktype, lang, blockname
        self.assertRaises(ModToolException, test_obj.run)
        test_obj.info['blocktype'] = 'general'
        # missing lang, blockname
        self.assertRaises(ModToolException, test_obj.run)
        test_obj.info['lang'] = 'python'
        test_obj.info['blockname'] = 'mul_ff'
        test_obj.add_py_qa = True
        test_obj.run()
        self.assertTrue(path.exists(
            path.join(module_dir, 'python', 'howto', 'mul_ff.py')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'python', 'howto', 'qa_mul_ff.py')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'grc', 'howto_mul_ff.block.yml')))

    @unittest.skipIf(skip_pylint_test, 'pylint dependency missing, skip test')
    def test_pylint_add(self):
        """ Pylint tests for API function add """
        # skip tests if newmod command wasn't successful
        if self.f_newmod:
            raise unittest.SkipTest("setUp for API function 'add' failed")
        module_dir = path.join(self.test_dir, 'gr-howto')

        ## The check for object instantiation ##
        test_obj = ModToolAdd()
        test_obj.dir = module_dir
        # missing blocktype, lang, blockname
        self.assertRaises(ModToolException, test_obj.run)
        test_obj.info['blocktype'] = 'general'
        # missing lang, blockname
        self.assertRaises(ModToolException, test_obj.run)
        test_obj.info['lang'] = 'python'
        test_obj.info['blockname'] = 'mul_ff'
        test_obj.add_py_qa = True
        test_obj.run()
        self.assertTrue(path.exists(
            path.join(module_dir, 'python', 'howto', 'mul_ff.py')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'python', 'howto', 'qa_mul_ff.py')))

        ## pylint tests ##
        python_dir = path.join(module_dir, 'python', 'howto')
        py_module = path.join(python_dir, 'mul_ff.py')
        (pylint_stdout, pylint_stderr) = py_run(py_module +
                                                ' --errors-only --disable=E0602', return_std=True)
        print(pylint_stdout.getvalue(), end='')
        py_module = path.join(python_dir, 'qa_mul_ff.py')
        (pylint_stdout, pylint_stderr) = py_run(
            py_module + ' --errors-only', return_std=True)
        print(pylint_stdout.getvalue(), end='')

    def test_rename(self):
        """ Tests for the API function rename """
        if self.f_newmod or self.f_add:
            raise unittest.SkipTest("setUp for API function 'rename' failed")

        module_dir = path.join(self.test_dir, 'gr-howto')
        test_dict = {}
        test_dict['directory'] = module_dir
        # Missing 2 arguments blockname, new_name
        self.assertRaises(ModToolException, ModToolRename(**test_dict).run)

        test_dict['blockname'] = 'square_ff'
        # Missing argument new_name
        self.assertRaises(ModToolException, ModToolRename(**test_dict).run)

        test_dict['new_name'] = '//#'
        # Invalid new block name!
        expected_message = "Invalid new block name '//#': names can only contain letters, numbers and underscores"
        self.assertRaisesRegex(
            ModToolException, expected_message, ModToolRename(**test_dict).run)

        test_dict['new_name'] = 'non-alpha-chars-like-hyphen-are-not-allowed'
        # Invalid new block name!
        self.assertRaises(ModToolException, ModToolRename(**test_dict).run)

        test_dict['new_name'] = None
        # New Block name not specified
        self.assertRaises(ModToolException, ModToolRename(**test_dict).run)

        ## Some tests for checking the renamed files ##
        test_dict['new_name'] = 'div_ff'
        ModToolRename(**test_dict).run()
        self.assertTrue(path.exists(
            path.join(module_dir, 'lib', 'div_ff_impl.h')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'lib', 'div_ff_impl.cc')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'python', 'howto', 'qa_div_ff.py')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'grc', 'howto_div_ff.block.yml')))

        ## The check for object instantiation ##
        test_obj = ModToolRename()
        test_obj.info['oldname'] = 'div_ff'
        test_obj.info['newname'] = 'sub_ff'
        test_obj.run()
        self.assertTrue(path.exists(
            path.join(module_dir, 'lib', 'sub_ff_impl.h')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'lib', 'sub_ff_impl.cc')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'python', 'howto', 'qa_sub_ff.py')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'grc', 'howto_sub_ff.block.yml')))

    def test_remove(self):
        """ Tests for the API function remove """
        if self.f_newmod or self.f_add:
            raise unittest.SkipTest("setUp for API function 'remove' failed")
        module_dir = path.join(self.test_dir, 'gr-howto')
        test_dict = {}
        # missing argument blockname
        self.assertRaises(ModToolException, ModToolRename(**test_dict).run)
        test_dict['directory'] = module_dir
        self.assertRaises(ModToolException, ModToolRename(**test_dict).run)

        ## Some tests to check blocks are not removed with different blocknames ##
        test_dict['blockname'] = 'div_ff'
        ModToolRemove(**test_dict).run()
        self.assertTrue(path.exists(
            path.join(module_dir, 'lib', 'square_ff_impl.h')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'lib', 'square_ff_impl.cc')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'python', 'howto', 'qa_square_ff.py')))
        self.assertTrue(path.exists(
            path.join(module_dir, 'grc', 'howto_square_ff.block.yml')))

        ## Some tests for checking the non-existence of removed files ##
        test_dict['blockname'] = 'square_ff'
        ModToolRemove(**test_dict).run()
        self.assertTrue(not path.exists(
            path.join(module_dir, 'lib', 'square_ff_impl.h')))
        self.assertTrue(not path.exists(
            path.join(module_dir, 'lib', 'square_ff_impl.cc')))
        self.assertTrue(not path.exists(
            path.join(module_dir, 'python', 'howto', 'qa_square_ff.py')))
        self.assertTrue(not path.exists(
            path.join(module_dir, 'grc', 'howto_square_ff.block.yml')))

    def test_makeyaml(self):
        """ Tests for the API function makeyaml """
        if self.f_newmod or self.f_add:
            raise unittest.SkipTest("setUp for API function 'makeyaml' failed")
        module_dir = path.join(self.test_dir, 'gr-howto')
        test_dict = {}
        # missing argument blockname
        self.assertRaises(ModToolException, ModToolMakeYAML(**test_dict).run)
        test_dict['directory'] = module_dir
        self.assertRaises(ModToolException, ModToolMakeYAML(**test_dict).run)

        ## Some tests to check if the command reuns ##
        test_dict['blockname'] = 'square_ff'
        ModToolMakeYAML(**test_dict).run()

    def test_disable(self):
        """ Tests for the API function disable """
        if self.f_newmod or self.f_add:
            raise unittest.SkipTest("setUp for API function 'disable' failed")
        module_dir = path.join(self.test_dir, 'gr-howto')
        test_dict = {}
        # missing argument blockname
        self.assertRaises(ModToolException, ModToolDisable(**test_dict).run)
        test_dict['directory'] = module_dir
        self.assertRaises(ModToolException, ModToolDisable(**test_dict).run)

        ## Some tests to check if the command reuns ##
        test_dict['blockname'] = 'square_ff'
        ModToolDisable(**test_dict).run()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
#
# Copyright 2005,2007,2010,2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from gnuradio import gr, gr_unittest, blocks


def calc_expected_result(src_data, n):
    assert (len(src_data) % n) == 0
    result = [list() for x in range(n)]
    # print "len(result) =", len(result)
    for i in range(len(src_data)):
        (result[i % n]).append(src_data[i])
    return result


class test_pipe_fittings(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_001(self):
        """
        Test stream_to_streams.
        """
        n = 8
        src_len = n * 8
        src_data = list(range(src_len))

        expected_results = calc_expected_result(src_data, n)
        # print "expected results: ", expected_results
        src = blocks.vector_source_i(src_data)
        op = blocks.stream_to_streams(gr.sizeof_int, n)
        self.tb.connect(src, op)

        dsts = []
        for i in range(n):
            dst = blocks.vector_sink_i()
            self.tb.connect((op, i), (dst, 0))
            dsts.append(dst)

        self.tb.run()

        for d in range(n):
            self.assertEqual(expected_results[d], dsts[d].data())

    def test_002(self):

        # Test streams_to_stream (using stream_to_streams).

        n = 8
        src_len = n * 8
        src_data = list(range(src_len))
        expected_results = src_data

        src = blocks.vector_source_i(src_data)
        op1 = blocks.stream_to_streams(gr.sizeof_int, n)
        op2 = blocks.streams_to_stream(gr.sizeof_int, n)
        dst = blocks.vector_sink_i()

        self.tb.connect(src, op1)
        for i in range(n):
            self.tb.connect((op1, i), (op2, i))
        self.tb.connect(op2, dst)

        self.tb.run()
        self.assertEqual(expected_results, dst.data())

    def test_003(self):

        # Test streams_to_vector (using stream_to_streams & vector_to_stream).

        n = 8
        src_len = n * 8
        src_data = list(range(src_len))
        expected_results = src_data

        src = blocks.vector_source_i(src_data)
        op1 = blocks.stream_to_streams(gr.sizeof_int, n)
        op2 = blocks.streams_to_vector(gr.sizeof_int, n)
        op3 = blocks.vector_to_stream(gr.sizeof_int, n)
        dst = blocks.vector_sink_i()

        self.tb.connect(src, op1)
        for i in range(n):
            self.tb.connect((op1, i), (op2, i))
        self.tb.connect(op2, op3, dst)

        self.tb.run()
        self.assertEqual(expected_results, dst.data())

    def test_004(self):

        # Test vector_to_streams.

        n = 8
        src_len = n * 8
        src_data = list(range(src_len))
        expected_results = src_data

        src = blocks.vector_source_i(src_data)
        op1 = blocks.stream_to_vector(gr.sizeof_int, n)
        op2 = blocks.vector_to_streams(gr.sizeof_int, n)
        op3 = blocks.streams_to_stream(gr.sizeof_int, n)
        dst = blocks.vector_sink_i()

        self.tb.connect(src, op1, op2)
        for i in range(n):
            self.tb.connect((op2, i), (op3, i))
        self.tb.connect(op3, dst)

        self.tb.run()
        self.assertEqual(expected_results, dst.data())


if __name__ == '__main__':
    gr_unittest.run(test_pipe_fittings)



#!/usr/bin/env python
#
# Copyright 2005,2008,2010 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
gr-uhd sanity checking
"""

from gnuradio import gr, gr_unittest, uhd


class test_uhd(gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000_nop(self):
        """Just see if we can import the module...
        They may not have a UHD device connected, etc.  Don't try to run anything"""
        pass

    def test_time_spec_t(self):
        seconds = 42.0
        time = uhd.time_spec_t(seconds)
        twice_time = time + time
        zero_time = time - time
        self.assertEqual(time.get_real_secs() * 2, seconds * 2)
        self.assertEqual(time.get_real_secs() - time.get_real_secs(), 0.0)

    def test_stream_args_channel_foo(self):
        """
        Try to manipulate the stream args channels for proper swig'ing checks.
        """
        # FYI: stream_args_t.channels.append does not work due to copy operation of STL vectors
        sa = uhd.stream_args_t()
        sa.channels = [1, 0]
        print(sa.channels)
        self.assertEqual(len(sa.channels), 2)
        self.assertEqual(sa.channels[0], 1)
        self.assertEqual(sa.channels[1], 0)


if __name__ == '__main__':
    gr_unittest.run(test_uhd)



#!/usr/bin/env python
#
# Copyright 2018 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

import os
import tempfile
import array
from gnuradio import gr, gr_unittest, blocks


class test_file_sink(gr_unittest.TestCase):

    def setUp(self):
        os.environ['GR_CONF_CONTROLPORT_ON'] = 'False'
        self.tb = gr.top_block()
        temp = tempfile.NamedTemporaryFile(delete=False)
        temp.close()
        self._datafilename = temp.name

    def tearDown(self):
        self.tb = None
        os.unlink(self._datafilename)

    def test_file_sink(self):
        data = range(1000)
        expected_result = data

        src = blocks.vector_source_f(data)
        snk = blocks.file_sink(gr.sizeof_float, self._datafilename)
        snk.set_unbuffered(True)
        self.tb.connect(src, snk)
        self.tb.run()
        snk.close()

        # Check file length (float: 4 * nsamples)
        file_size = os.stat(self._datafilename).st_size
        self.assertEqual(file_size, 4 * len(data))

        # Check file contents
        result_data = array.array('f')
        with open(self._datafilename, 'rb') as datafile:
            result_data.fromfile(datafile, len(data))
        self.assertFloatTuplesAlmostEqual(expected_result, result_data)


if __name__ == '__main__':
    gr_unittest.run(test_file_sink)



#
# Copyright 2010-2012 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#

'''
Provides source and sink blocks to interface with the UHD library.
Used to send and receive data between the Ettus Research, LLC product
line.
'''

########################################################################
# Prepare uhd swig module to make it more pythonic
########################################################################


def _prepare_uhd_python():
    try:
        from . import uhd_python
    except ImportError:
        import os
        dirname, filename = os.path.split(os.path.abspath(__file__))
        __path__.append(os.path.join(dirname, "bindings"))
        from . import uhd_python

    # some useful typedefs for the user
    setattr(uhd_python, 'freq_range_t', uhd_python.meta_range_t)
    setattr(uhd_python, 'gain_range_t', uhd_python.meta_range_t)

    # Make the python tune request object inherit from float
    # so that it can be passed in GRC as a frequency parameter.
    # The type checking in GRC will accept the tune request.
    # Also use kwargs to construct individual struct elements.
    class tune_request_t(uhd_python.tune_request_t):
        # def __new__(self, *args, **kwargs): return float.__new__(self)
        def __float__(self): return self.target_freq

        def __init__(self, *args, **kwargs):
            super().__init__(*args)
            for key, val in list(kwargs.items()):
                setattr(self, key, val)
    setattr(uhd_python, 'tune_request_t', tune_request_t)

    # handle general things on all uhd_python attributes
    # Install the __str__ and __repr__ handlers if applicable
    # Create aliases for uhd swig attributes to avoid the "_t"
    for attr in dir(uhd_python):
        myobj = getattr(uhd_python, attr)
        if hasattr(myobj, 'to_string'):
            myobj.__repr__ = lambda o: o.to_string().strip()
        if hasattr(myobj, 'to_pp_string'):
            myobj.__str__ = lambda o: o.to_pp_string().strip()
        if hasattr(myobj, 'to_bool'):
            myobj.__nonzero__ = lambda o: o.to_bool()
        if hasattr(myobj, 'to_int'):
            myobj.__int__ = lambda o: o.to_int()
        if hasattr(myobj, 'to_real'):
            myobj.__float__ = lambda o: o.to_real()
        if attr.endswith('_t'):
            setattr(uhd_python, attr[:-2], myobj)

    # make a new find devices that casts everything with the pythonized device_addr_t which has __str__
    def find_devices(*args, **kwargs):
        def to_pythonized_dev_addr(dev_addr):
            new_dev_addr = uhd_python.device_addr_t()
            for key in list(dev_addr.keys()):
                new_dev_addr[key] = dev_addr.get(key)
            return new_dev_addr
        return __builtins__['map'](to_pythonized_dev_addr, uhd_python.find_devices_raw(*args, **kwargs))
    setattr(uhd_python, 'find_devices', find_devices)


########################################################################
# Initialize this module with the contents of uhd pybind
########################################################################
_prepare_uhd_python()

from .uhd_python import *

try:
    from .replaymsgpushbutton import ReplayMsgPushButton
except ModuleNotFoundError:
    # This Button uses PyQt5 and is only used in GUI applications that use the replay block;
    # the rest of gr-uhd python functionality should not be limited by PyQt5 being installed,
    # so ignore if the module is not found
    pass



#!/usr/bin/env python
# Copyright 2012,2013 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


import numpy

from gnuradio import gr, gr_unittest, digital, blocks
import pmt


class qa_ofdm_frame_equalizer_vcvc (gr_unittest.TestCase):

    def setUp(self):
        self.tb = gr.top_block()
        self.tsb_key = "tsb_key"

    def tearDown(self):
        self.tb = None

    def test_001_simple(self):
        """ Very simple functionality testing:
        - static equalizer
        - init channel state with all ones
        - transmit all ones
        - make sure we rx all ones
        - Tag check: put in frame length tag and one other random tag,
                     make sure they're propagated
        """
        fft_len = 8
        equalizer = digital.ofdm_equalizer_static(fft_len)
        n_syms = 3
        tx_data = [1, ] * fft_len * n_syms
        chan_tag = gr.tag_t()
        chan_tag.offset = 0
        chan_tag.key = pmt.string_to_symbol("ofdm_sync_chan_taps")
        chan_tag.value = pmt.init_c32vector(fft_len, (1,) * fft_len)
        random_tag = gr.tag_t()
        random_tag.offset = 1
        random_tag.key = pmt.string_to_symbol("foo")
        random_tag.value = pmt.from_long(42)
        src = blocks.vector_source_c(
            tx_data, False, fft_len, (chan_tag, random_tag))
        eq = digital.ofdm_frame_equalizer_vcvc(
            equalizer.base(), 0, self.tsb_key)
        sink = blocks.tsb_vector_sink_c(fft_len, tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                fft_len,
                n_syms,
                self.tsb_key),
            eq,
            sink)
        self.tb.run()
        # Check data
        self.assertEqual(tx_data, sink.data()[0])
        # Check tags
        tag_dict = dict()
        for tag in sink.tags():
            ptag = gr.tag_to_python(tag)
            tag_dict[ptag.key] = ptag.value
        expected_dict = {
            'foo': 42
        }
        self.assertEqual(tag_dict, expected_dict)

    def test_001b_simple_skip_nothing(self):
        """
        Same as before, but put a skip-header in there
        """
        fft_len = 8
        equalizer = digital.ofdm_equalizer_static(fft_len, symbols_skipped=1)
        n_syms = 3
        tx_data = [1, ] * fft_len * n_syms
        chan_tag = gr.tag_t()
        chan_tag.offset = 0
        chan_tag.key = pmt.string_to_symbol("ofdm_sync_chan_taps")
        chan_tag.value = pmt.init_c32vector(fft_len, (1,) * fft_len)
        src = blocks.vector_source_c(tx_data, False, fft_len, (chan_tag,))
        eq = digital.ofdm_frame_equalizer_vcvc(
            equalizer.base(), 0, self.tsb_key)
        sink = blocks.tsb_vector_sink_c(fft_len, tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                fft_len,
                n_syms,
                self.tsb_key),
            eq,
            sink)
        self.tb.run()
        # Check data
        self.assertEqual(tx_data, sink.data()[0])

    def test_001c_carrier_offset_no_cp(self):
        """
        Same as before, but put a carrier offset in there
        """
        fft_len = 8
        cp_len = 0
        n_syms = 1
        carr_offset = 1
        occupied_carriers = ((-2, -1, 1, 2),)
        tx_data = (
            0, 0, 0, -1j, -1j, 0, -1j, -1j,
        )
        # The rx'd signal is shifted
        rx_expected = (0, 0, 1, 1, 0, 1, 1, 0) * n_syms
        equalizer = digital.ofdm_equalizer_static(fft_len, occupied_carriers)
        chan_tag = gr.tag_t()
        chan_tag.offset = 0
        chan_tag.key = pmt.string_to_symbol("ofdm_sync_chan_taps")
        # Note: this is shifted to the correct position!
        chan_tag.value = pmt.init_c32vector(
            fft_len, (0, 0, -1j, -1j, 0, -1j, -1j, 0))
        offset_tag = gr.tag_t()
        offset_tag.offset = 0
        offset_tag.key = pmt.string_to_symbol("ofdm_sync_carr_offset")
        offset_tag.value = pmt.from_long(carr_offset)
        src = blocks.vector_source_c(
            tx_data, False, fft_len, (chan_tag, offset_tag))
        eq = digital.ofdm_frame_equalizer_vcvc(
            equalizer.base(), cp_len, self.tsb_key)
        sink = blocks.tsb_vector_sink_c(fft_len, tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                fft_len,
                n_syms,
                self.tsb_key),
            eq,
            sink)
        self.tb.run()
        # Check data
        self.assertComplexTuplesAlmostEqual(
            rx_expected, sink.data()[0], places=4)

    def test_001c_carrier_offset_cp(self):
        """
        Same as before, but put a carrier offset in there and a CP
        """
        fft_len = 8
        cp_len = 2
        n_syms = 3
        # cp_len/fft_len == 1/4, therefore, the phase is rotated by
        # carr_offset * \pi/2 in every symbol
        occupied_carriers = ((-2, -1, 1, 2),)
        carr_offset = -1
        tx_data = (
            0, -1j, -1j, 0, -1j, -1j, 0, 0,
            0, -1, -1, 0, -1, -1, 0, 0,
            0, 1j, 1j, 0, 1j, 1j, 0, 0,
        )
        # Rx'd signal is corrected
        rx_expected = (0, 0, 1, 1, 0, 1, 1, 0) * n_syms
        equalizer = digital.ofdm_equalizer_static(fft_len, occupied_carriers)
        chan_tag = gr.tag_t()
        chan_tag.offset = 0
        chan_tag.key = pmt.string_to_symbol("ofdm_sync_chan_taps")
        chan_tag.value = pmt.init_c32vector(fft_len, (0, 0, 1, 1, 0, 1, 1, 0))
        offset_tag = gr.tag_t()
        offset_tag.offset = 0
        offset_tag.key = pmt.string_to_symbol("ofdm_sync_carr_offset")
        offset_tag.value = pmt.from_long(carr_offset)
        src = blocks.vector_source_c(
            tx_data, False, fft_len, (chan_tag, offset_tag))
        eq = digital.ofdm_frame_equalizer_vcvc(
            equalizer.base(), cp_len, self.tsb_key)
        sink = blocks.tsb_vector_sink_c(fft_len, tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                fft_len,
                n_syms,
                self.tsb_key),
            eq,
            sink)
        self.tb.run()
        # Check data
        self.assertComplexTuplesAlmostEqual(
            rx_expected, sink.data()[0], places=4)

    def test_002_static(self):
        """
        - Add a simple channel
        - Make symbols QPSK
        """
        fft_len = 8
        #           4   5  6  7   0  1  2   3
        tx_data = [-1, -1, 1, 2, -1, 3, 0, -1,  # 0
                   -1, -1, 0, 2, -1, 2, 0, -1,  # 8
                   -1, -1, 3, 0, -1, 1, 0, -1,  # 16 (Pilot symbols)
                   -1, -1, 1, 1, -1, 0, 2, -1]  # 24
        cnst = digital.constellation_qpsk()
        tx_signal = [
            cnst.map_to_points_v(x)[0] if x != -
            1 else 0 for x in tx_data]
        occupied_carriers = ((1, 2, 6, 7),)
        pilot_carriers = ((), (), (1, 2, 6, 7), ())
        pilot_symbols = (
            [], [], [cnst.map_to_points_v(x)[0] for x in (1, 0, 3, 0)], []
        )
        equalizer = digital.ofdm_equalizer_static(
            fft_len, occupied_carriers, pilot_carriers, pilot_symbols)
        channel = [
            0, 0, 1, 1, 0, 1, 1, 0,
            # These coefficients will be rotated slightly (but less than \pi/2)
            0, 0, 1, 1, 0, 1, 1, 0,
            0, 0, 1j, 1j, 0, 1j, 1j, 0,  # Go crazy here!
            0, 0, 1j, 1j, 0, 1j, 1j, 0
        ]
        for idx in range(fft_len, 2 * fft_len):
            channel[idx] = channel[idx - fft_len] * \
                numpy.exp(1j * .1 * numpy.pi * (numpy.random.rand() - .5))
        chan_tag = gr.tag_t()
        chan_tag.offset = 0
        chan_tag.key = pmt.string_to_symbol("ofdm_sync_chan_taps")
        chan_tag.value = pmt.init_c32vector(fft_len, channel[:fft_len])
        src = blocks.vector_source_c(numpy.multiply(
            tx_signal, channel), False, fft_len, (chan_tag,))
        sink = blocks.tsb_vector_sink_c(vlen=fft_len, tsb_key=self.tsb_key)
        eq = digital.ofdm_frame_equalizer_vcvc(
            equalizer.base(), 0, self.tsb_key, True)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                fft_len,
                len(tx_data) //
                fft_len,
                self.tsb_key),
            eq,
            sink)
        self.tb.run()
        rx_data = [
            cnst.decision_maker_v(
                (x,)) if x != 0 else -1 for x in sink.data()[0]]
        # Check data
        self.assertEqual(tx_data, rx_data)
        # Check tags
        tag_dict = dict()
        for tag in sink.tags():
            ptag = gr.tag_to_python(tag)
            tag_dict[ptag.key] = ptag.value
            if ptag.key == 'ofdm_sync_chan_taps':
                tag_dict[ptag.key] = list(pmt.c32vector_elements(tag.value))
            else:
                tag_dict[ptag.key] = pmt.to_python(tag.value)
        expected_dict = {
            'ofdm_sync_chan_taps': channel[-fft_len:]
        }
        # Won't be exactly the same when compiled with -fcx-limited-range
        self.assertTrue(
            numpy.allclose(
                tag_dict['ofdm_sync_chan_taps'],
                expected_dict['ofdm_sync_chan_taps']))
        expected_dict['ofdm_sync_chan_taps'] = tag_dict['ofdm_sync_chan_taps']
        self.assertEqual(tag_dict, expected_dict)

    def test_002_static_wo_tags(self):
        """ Same as before, but the input stream has no tag.
        We specify the frame size in the constructor.
        We also specify a tag key, so the output stream *should* have
        a TSB tag.
        """
        fft_len = 8
        n_syms = 4
        #           4   5  6  7   0  1  2   3
        tx_data = [-1, -1, 1, 2, -1, 3, 0, -1,  # 0
                   -1, -1, 0, 2, -1, 2, 0, -1,  # 8
                   -1, -1, 3, 0, -1, 1, 0, -1,  # 16 (Pilot symbols)
                   -1, -1, 1, 1, -1, 0, 2, -1]  # 24
        cnst = digital.constellation_qpsk()
        tx_signal = [
            cnst.map_to_points_v(x)[0] if x != -
            1 else 0 for x in tx_data]
        occupied_carriers = ((1, 2, 6, 7),)
        pilot_carriers = ((), (), (1, 2, 6, 7), ())
        pilot_symbols = (
            [], [], [cnst.map_to_points_v(x)[0] for x in (1, 0, 3, 0)], []
        )
        equalizer = digital.ofdm_equalizer_static(
            fft_len, occupied_carriers, pilot_carriers, pilot_symbols)
        channel = [
            0, 0, 1, 1, 0, 1, 1, 0,
            # These coefficients will be rotated slightly (below)...
            0, 0, 1, 1, 0, 1, 1, 0,
            0, 0, 1j, 1j, 0, 1j, 1j, 0,  # Go crazy here!
            0, 0, 1j, 1j, 0, 1j, 1j, 0  # ...and again here.
        ]
        for idx in range(fft_len, 2 * fft_len):
            channel[idx] = channel[idx - fft_len] * \
                numpy.exp(1j * .1 * numpy.pi * (numpy.random.rand() - .5))
            idx2 = idx + 2 * fft_len
            channel[idx2] = channel[idx2] * \
                numpy.exp(1j * 0 * numpy.pi * (numpy.random.rand() - .5))
        src = blocks.vector_source_c(
            numpy.multiply(
                tx_signal,
                channel),
            False,
            fft_len)
        eq = digital.ofdm_frame_equalizer_vcvc(
            equalizer.base(), 0, self.tsb_key, False, n_syms)
        sink = blocks.tsb_vector_sink_c(vlen=fft_len, tsb_key=self.tsb_key)
        self.tb.connect(
            src,
            blocks.stream_to_tagged_stream(
                gr.sizeof_gr_complex,
                fft_len,
                len(tx_data) //
                fft_len,
                self.tsb_key),
            eq,
            sink)
        self.tb.run()
        rx_data = [
            cnst.decision_maker_v(
                (x,)) if x != 0 else -1 for x in sink.data()[0]]
        self.assertEqual(tx_data, rx_data)
        # Check TSB Functionality
        packets = sink.data()
        self.assertEqual(len(packets), 1)
        self.assertEqual(len(packets[0]), len(tx_data))

    def test_002_static_wo_tags_2(self):
        fft_len = 8
        #           4   5  6  7   0  1  2   3
        tx_data = [-1, -1, 1, 2, -1, 3, 0, -1,  # 0
                   -1, -1, 0, 2, -1, 2, 0, -1,  # 8
                   -1, -1, 3, 0, -1, 1, 0, -1,  # 16 (Pilot symbols)
                   -1, -1, 1, 1, -1, 0, 2, -1]  # 24
        cnst = digital.constellation_qpsk()
        tx_signal = [
            cnst.map_to_points_v(x)[0] if x != -
            1 else 0 for x in tx_data]
        occupied_carriers = ((1, 2, 6, 7),)
        pilot_carriers = ((), (), (1, 2, 6, 7), ())
        pilot_symbols = (
            [], [], [cnst.map_to_points_v(x)[0] for x in (1, 0, 3, 0)], []
        )
        equalizer = digital.ofdm_equalizer_static(
            fft_len, occupied_carriers, pilot_carriers, pilot_symbols)
        channel = [
            0, 0, 1, 1, 0, 1, 1, 0,
            # These coefficients will be rotated slightly...
            0, 0, 1, 1, 0, 1, 1, 0,
            0, 0, 1j, 1j, 0, 1j, 1j, 0,  # Go crazy here!
            0, 0, 1j, 1j, 0, 1j, 1j, 0  # ...and again here.
        ]
        for idx in range(fft_len, 2 * fft_len):
            channel[idx] = channel[idx - fft_len] * \
                numpy.exp(1j * .1 * numpy.pi * (numpy.random.rand() - .5))
            idx2 = idx + 2 * fft_len
            channel[idx2] = channel[idx2] * \
                numpy.exp(1j * 0 * numpy.pi * (numpy.random.rand() - .5))
        src = blocks.vector_source_c(
            numpy.multiply(
                tx_signal,
                channel),
            False,
            fft_len)
        sink = blocks.vector_sink_c(fft_len)
        eq = digital.ofdm_frame_equalizer_vcvc(
            equalizer.base(), 0, "", False, 4)
        self.tb.connect(src, eq, sink)
        self.tb.run()
        rx_data = [
            cnst.decision_maker_v(
                (x,)) if x != 0 else -1 for x in sink.data()]
        self.assertEqual(tx_data, rx_data)

    def test_002_simpledfe(self):
        """ Use the simple DFE equalizer. """
        fft_len = 8
        #           4   5  6  7   0  1  2   3
        tx_data = [-1, -1, 1, 2, -1, 3, 0, -1,  # 0
                   -1, -1, 0, 2, -1, 2, 0, -1,  # 8
                   -1, -1, 3, 0, -1, 1, 0, -1,  # 16 (Pilot symbols)
                   -1, -1, 1, 1, -1, 0, 2, -1]  # 24
        cnst = digital.constellation_qpsk()
        tx_signal = [
            cnst.map_to_points_v(x)[0] if x != -
            1 else 0 for x in tx_data]
        occupied_carriers = ((1, 2, 6, 7),)
        pilot_carriers = ((), (), (1, 2, 6, 7), ())
        pilot_symbols = (
            [], [], [cnst.map_to_points_v(x)[0] for x in (1, 0, 3, 0)], []
        )
        equalizer = digital.ofdm_equalizer_simpledfe(
            fft_len,
            cnst.base(),
            occupied_carriers,
            pilot_carriers,
            pilot_symbols,
            0,
            0.01)
        equalizer_soft = digital.ofdm_equalizer_simpledfe(
            fft_len,
            cnst.base(),
            occupied_carriers,
            pilot_carriers,
            pilot_symbols,
            0,
            0.01,
            enable_soft_output=True)
        channel = [
            0, 0, 1, 1, 0, 1, 1, 0,
            # These coefficients will be rotated slightly...
            0, 0, 1, 1, 0, 1, 1, 0,
            0, 0, 1j, 1j, 0, 1j, 1j, 0,  # Go crazy here!
            0, 0, 1j, 1j, 0, 1j, 1j, 0  # ...and again here.
        ]
        for idx in range(fft_len, 2 * fft_len):
            channel[idx] = channel[idx - fft_len] * \
                numpy.exp(1j * .1 * numpy.pi * (numpy.random.rand() - .5))
            idx2 = idx + 2 * fft_len
            channel[idx2] = channel[idx2] * \
                numpy.exp(1j * 0 * numpy.pi * (numpy.random.rand() - .5))
        chan_tag = gr.tag_t()
        chan_tag.offset = 0
        chan_tag.key = pmt.string_to_symbol("ofdm_sync_chan_taps")
        chan_tag.value = pmt.init_c32vector(fft_len, channel[:fft_len])
        src = blocks.vector_source_c(numpy.multiply(
            tx_signal, channel), False, fft_len, (chan_tag,))
        eq = digital.ofdm_frame_equalizer_vcvc(
            equalizer.base(), 0, self.tsb_key, True)
        eq_soft = digital.ofdm_frame_equalizer_vcvc(
            equalizer_soft.base(), 0, self.tsb_key, True)
        sink = blocks.tsb_vector_sink_c(fft_len, tsb_key=self.tsb_key)
        sink_soft = blocks.tsb_vector_sink_c(fft_len, tsb_key=self.tsb_key)
        stream_to_tagged = blocks.stream_to_tagged_stream(
            gr.sizeof_gr_complex, fft_len, len(tx_data) // fft_len, self.tsb_key)
        self.tb.connect(
            src,
            stream_to_tagged,
            eq,
            sink
        )
        self.tb.connect(
            stream_to_tagged,
            eq_soft,
            sink_soft
        )
        self.tb.run()

        out_syms = numpy.array(sink.data()[0])
        out_syms_soft = numpy.array(sink_soft.data()[0])

        def demod(syms): return [
            cnst.decision_maker_v(
                (x,)) if x != 0 else -1 for x in syms]
        rx_data = demod(out_syms)
        rx_data_soft = demod(out_syms_soft)

        # Uncomment to plot symbols
        #import matplotlib.pyplot as plt
        #def plot_syms(d): plt.figure(); plt.plot(d.real, d.imag, 'b.')
        #
        # plot_syms(out_syms)
        # plot_syms(out_syms_soft)
        # plt.show()

        self.assertEqual(tx_data, rx_data)
        self.assertEqual(rx_data, rx_data_soft)
        self.assertFalse(numpy.allclose(out_syms, out_syms_soft))
        self.assertEqual(len(sink.tags()), 1)
        tag = sink.tags()[0]
        self.assertEqual(pmt.symbol_to_string(tag.key), "ofdm_sync_chan_taps")
        self.assertComplexTuplesAlmostEqual(
            list(pmt.c32vector_elements(tag.value)), channel[-fft_len:], places=1)


if __name__ == '__main__':
    gr_unittest.run(qa_ofdm_frame_equalizer_vcvc)



#!/usr/bin/env python
#
# Copyright 2014 Free Software Foundation, Inc.
#
# This file is part of GNU Radio
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
#


from collections import deque

from gnuradio import gr, gr_unittest, blocks, fec


class test_puncture (gr_unittest.TestCase):

    def puncture_setup(self):
        p = []
        for i in range(self.puncsize):
            p.append(self.puncpat >> (self.puncsize - 1 - i) & 1)
        d = deque(p)
        d.rotate(self.delay)
        _puncpat = list(d)

        self.expected = []
        for n in range(len(self.src_data) // self.puncsize):
            for i in range(self.puncsize):
                if _puncpat[i] == 1:
                    self.expected.append(self.src_data[n * self.puncsize + i])

    def setUp(self):
        self.src_data = 10000 * list(range(64))
        self.tb = gr.top_block()

    def tearDown(self):
        self.tb = None

    def test_000(self):
        # Test normal operation of the puncture block

        self.puncsize = 8
        self.puncpat = 0xEF
        self.delay = 0

        self.puncture_setup()

        src = blocks.vector_source_b(self.src_data)
        op = fec.puncture_bb(self.puncsize, self.puncpat, self.delay)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op, dst)
        self.tb.run()

        dst_data = list(dst.data())
        for i in range(len(dst_data)):
            dst_data[i] = int(dst_data[i])

        self.assertEqual(self.expected, dst_data)

    def test_001(self):
        # Test normal operation of the puncture block with a delay

        self.puncsize = 8
        self.puncpat = 0xEE
        self.delay = 1

        self.src_data = list(range(16))

        self.puncture_setup()

        src = blocks.vector_source_b(self.src_data)
        op = fec.puncture_bb(self.puncsize, self.puncpat, self.delay)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op, dst)
        self.tb.run()

        dst_data = list(dst.data())
        for i in range(len(dst_data)):
            dst_data[i] = int(dst_data[i])

        self.assertEqual(self.expected, dst_data)

    def test_002(self):
        # Test scenario where we have defined a puncture pattern with
        # more bits than the puncsize.

        self.puncsize = 4
        self.puncpat = 0x5555
        self.delay = 0

        self.puncture_setup()

        src = blocks.vector_source_b(self.src_data)
        op = fec.puncture_bb(self.puncsize, self.puncpat, self.delay)
        dst = blocks.vector_sink_b()

        self.tb.connect(src, op, dst)
        self.tb.run()

        dst_data = list(dst.data())
        for i in range(len(dst_data)):
            dst_data[i] = int(dst_data[i])

        self.assertEqual(self.expected, dst_data)

    def test_003(self):
        # Test scenario where we have defined a puncture pattern with
        # more bits than the puncsize with a delay. The python code
        # doesn't account for this when creating self.expected, but
        # this should be equivalent to a puncpat of the correct size.

        self.puncsize = 4
        self.puncpat0 = 0x5555  # too many bits set
        self.puncpat1 = 0x55   # num bits = puncsize
        self.delay = 1

        src = blocks.vector_source_b(self.src_data)
        op0 = fec.puncture_bb(self.puncsize, self.puncpat0, self.delay)
        op1 = fec.puncture_bb(self.puncsize, self.puncpat1, self.delay)
        dst0 = blocks.vector_sink_b()
        dst1 = blocks.vector_sink_b()

        self.tb.connect(src, op0, dst0)
        self.tb.connect(src, op1, dst1)
        self.tb.run()

        dst_data0 = list(dst0.data())
        for i in range(len(dst_data0)):
            dst_data0[i] = int(dst_data0[i])

        dst_data1 = list(dst1.data())
        for i in range(len(dst_data1)):
            dst_data1[i] = int(dst_data1[i])

        self.assertEqual(dst_data1, dst_data0)

    def test_f_000(self):
        # Test normal operation of the float puncture block

        self.puncsize = 8
        self.puncpat = 0xEF
        self.delay = 0

        self.puncture_setup()

        src = blocks.vector_source_f(self.src_data)
        op = fec.puncture_ff(self.puncsize, self.puncpat, self.delay)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, op, dst)
        self.tb.run()

        dst_data = list(dst.data())
        self.assertEqual(self.expected, dst_data)

    def test_f_001(self):
        # Test normal operation of the puncture block with a delay

        self.puncsize = 8
        self.puncpat = 0xEE
        self.delay = 1

        self.src_data = list(range(16))

        self.puncture_setup()

        src = blocks.vector_source_f(self.src_data)
        op = fec.puncture_ff(self.puncsize, self.puncpat, self.delay)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, op, dst)
        self.tb.run()

        dst_data = list(dst.data())
        self.assertEqual(self.expected, dst_data)

    def test_f_002(self):
        # Test scenariou where we have defined a puncture pattern with
        # more bits than the puncsize.

        self.puncsize = 4
        self.puncpat = 0x5555
        self.delay = 0

        self.puncture_setup()

        src = blocks.vector_source_f(self.src_data)
        op = fec.puncture_ff(self.puncsize, self.puncpat, self.delay)
        dst = blocks.vector_sink_f()

        self.tb.connect(src, op, dst)
        self.tb.run()

        dst_data = list(dst.data())
        self.assertEqual(self.expected, dst_data)

    def test_f_003(self):
        # Test scenariou where we have defined a puncture pattern with
        # more bits than the puncsize with a delay. The python code
        # doesn't account for this when creating self.expected, but
        # this should be equivalent to a puncpat of the correct size.

        self.puncsize = 4
        self.puncpat0 = 0x5555  # too many bits set
        self.puncpat1 = 0x55   # num bits = puncsize
        self.delay = 1

        src = blocks.vector_source_f(self.src_data)
        op0 = fec.puncture_ff(self.puncsize, self.puncpat0, self.delay)
        op1 = fec.puncture_ff(self.puncsize, self.puncpat1, self.delay)
        dst0 = blocks.vector_sink_f()
        dst1 = blocks.vector_sink_f()

        self.tb.connect(src, op0, dst0)
        self.tb.connect(src, op1, dst1)
        self.tb.run()

        dst_data0 = list(dst0.data())
        dst_data1 = list(dst1.data())

        self.assertEqual(dst_data1, dst_data0)


if __name__ == '__main__':
    gr_unittest.run(test_puncture)
